@@ -1,10 +1,13 @@
+require 'bundler/setup'
 require 'active_record'
-ActiveRecord::Base.raise_in_transactional_callbacks = true if ActiveRecord::VERSION::STRING >= '4.2'
-
 require 'minitest/autorun'
+require 'paranoia'
+
 test_framework = defined?(MiniTest::Test) ? MiniTest::Test : MiniTest::Unit::TestCase
 
-require File.expand_path(File.dirname(__FILE__) + "/../lib/paranoia")
+if ActiveRecord::Base.respond_to?(:raise_in_transactional_callbacks=)
+  ActiveRecord::Base.raise_in_transactional_callbacks = true
+end
 
 def connect!
   ActiveRecord::Base.establish_connection :adapter => 'sqlite3', database: ':memory:'
@@ -20,7 +23,9 @@ def setup!
     'paranoid_model_with_build_belongs' => 'parent_model_id INTEGER, deleted_at DATETIME, paranoid_model_with_has_one_and_build_id INTEGER, name VARCHAR(32)',
     'paranoid_model_with_anthor_class_name_belongs' => 'parent_model_id INTEGER, deleted_at DATETIME, paranoid_model_with_has_one_id INTEGER',
     'paranoid_model_with_foreign_key_belongs' => 'parent_model_id INTEGER, deleted_at DATETIME, has_one_foreign_key_id INTEGER',
+    'paranoid_model_with_timestamps' => 'parent_model_id INTEGER, created_at DATETIME, updated_at DATETIME, deleted_at DATETIME',
     'not_paranoid_model_with_belongs' => 'parent_model_id INTEGER, paranoid_model_with_has_one_id INTEGER',
+    'not_paranoid_model_with_belongs_and_assocation_not_soft_destroyed_validator' => 'parent_model_id INTEGER, paranoid_model_with_has_one_id INTEGER',
     'paranoid_model_with_has_one_and_builds' => 'parent_model_id INTEGER, color VARCHAR(32), deleted_at DATETIME, has_one_foreign_key_id INTEGER',
     'featureful_models' => 'deleted_at DATETIME, name VARCHAR(32)',
     'plain_models' => 'deleted_at DATETIME',
@@ -28,7 +33,7 @@ def setup!
     'fail_callback_models' => 'deleted_at DATETIME',
     'related_models' => 'parent_model_id INTEGER, parent_model_with_counter_cache_column_id INTEGER, deleted_at DATETIME',
     'asplode_models' => 'parent_model_id INTEGER, deleted_at DATETIME',
-    'employers' => 'deleted_at DATETIME',
+    'employers' => 'name VARCHAR(32), deleted_at DATETIME',
     'employees' => 'deleted_at DATETIME',
     'jobs' => 'employer_id INTEGER NOT NULL, employee_id INTEGER NOT NULL, deleted_at DATETIME',
     'custom_column_models' => 'destroyed_at DATETIME',
@@ -37,6 +42,12 @@ def setup!
     'polymorphic_models' => 'parent_id INTEGER, parent_type STRING, deleted_at DATETIME',
     'namespaced_paranoid_has_ones' => 'deleted_at DATETIME, paranoid_belongs_tos_id INTEGER',
     'namespaced_paranoid_belongs_tos' => 'deleted_at DATETIME, paranoid_has_one_id INTEGER',
+    'unparanoid_unique_models' => 'name VARCHAR(32), paranoid_with_unparanoids_id INTEGER',
+    'active_column_models' => 'deleted_at DATETIME, active BOOLEAN',
+    'active_column_model_with_uniqueness_validations' => 'name VARCHAR(32), deleted_at DATETIME, active BOOLEAN',
+    'paranoid_model_with_belongs_to_active_column_model_with_has_many_relationships' => 'name VARCHAR(32), deleted_at DATETIME, active BOOLEAN, active_column_model_with_has_many_relationship_id INTEGER',
+    'active_column_model_with_has_many_relationships' => 'name VARCHAR(32), deleted_at DATETIME, active BOOLEAN', 
+    'without_default_scope_models' => 'deleted_at DATETIME'
   }.each do |table_name, columns_as_sql_string|
     ActiveRecord::Base.connection.execute "CREATE TABLE #{table_name} (id INTEGER NOT NULL PRIMARY KEY, #{columns_as_sql_string})"
   end
@@ -52,8 +63,15 @@ setup!
 
 class ParanoiaTest < test_framework
   def setup
-    ActiveRecord::Base.connection.tables.each do |table|
-      ActiveRecord::Base.connection.execute "DELETE FROM #{table}"
+    connection = ActiveRecord::Base.connection
+    cleaner = ->(source) {
+      ActiveRecord::Base.connection.execute "DELETE FROM #{source}"
+    }
+
+    if ActiveRecord::VERSION::MAJOR < 5
+      connection.tables.each(&cleaner)
+    else
+      connection.data_sources.each(&cleaner)
     end
   end
 
@@ -104,9 +122,9 @@ class ParanoiaTest < test_framework
     model.remove_called_variables     # clear called callback flags
     model.destroy
 
-    assert_equal nil, model.instance_variable_get(:@update_callback_called)
-    assert_equal nil, model.instance_variable_get(:@save_callback_called)
-    assert_equal nil, model.instance_variable_get(:@validate_called)
+    assert_nil model.instance_variable_get(:@update_callback_called)
+    assert_nil model.instance_variable_get(:@save_callback_called)
+    assert_nil model.instance_variable_get(:@validate_called)
 
     assert model.instance_variable_get(:@destroy_callback_called)
     assert model.instance_variable_get(:@after_destroy_callback_called)
@@ -120,11 +138,27 @@ class ParanoiaTest < test_framework
     model.remove_called_variables     # clear called callback flags
     model.delete
 
-    assert_equal nil, model.instance_variable_get(:@update_callback_called)
-    assert_equal nil, model.instance_variable_get(:@save_callback_called)
-    assert_equal nil, model.instance_variable_get(:@validate_called)
-    assert_equal nil, model.instance_variable_get(:@destroy_callback_called)
-    assert_equal nil, model.instance_variable_get(:@after_destroy_callback_called)
+    assert_nil model.instance_variable_get(:@update_callback_called)
+    assert_nil model.instance_variable_get(:@save_callback_called)
+    assert_nil model.instance_variable_get(:@validate_called)
+    assert_nil model.instance_variable_get(:@destroy_callback_called)
+    assert_nil model.instance_variable_get(:@after_destroy_callback_called)
+    assert_nil model.instance_variable_get(:@after_commit_callback_called)
+  end
+
+  def test_delete_in_transaction_behavior_for_plain_models_callbacks
+    model = CallbackModel.new
+    model.save
+    model.remove_called_variables     # clear called callback flags
+    CallbackModel.transaction do
+      model.delete
+    end
+
+    assert_nil model.instance_variable_get(:@update_callback_called)
+    assert_nil model.instance_variable_get(:@save_callback_called)
+    assert_nil model.instance_variable_get(:@validate_called)
+    assert_nil model.instance_variable_get(:@destroy_callback_called)
+    assert_nil model.instance_variable_get(:@after_destroy_callback_called)
     assert model.instance_variable_get(:@after_commit_callback_called)
   end
 
@@ -155,14 +189,30 @@ class ParanoiaTest < test_framework
     p2 = ParanoidModel.create(:parent_model => parent2)
     p1.destroy
     p2.destroy
+    
     assert_equal 0, parent1.paranoid_models.count
     assert_equal 1, parent1.paranoid_models.only_deleted.count
+
+    assert_equal 2, ParanoidModel.only_deleted.joins(:parent_model).count    
     assert_equal 1, parent1.paranoid_models.deleted.count
+    assert_equal 0, parent1.paranoid_models.without_deleted.count
     p3 = ParanoidModel.create(:parent_model => parent1)
     assert_equal 2, parent1.paranoid_models.with_deleted.count
+    assert_equal 1, parent1.paranoid_models.without_deleted.count
     assert_equal [p1,p3], parent1.paranoid_models.with_deleted
   end
 
+  def test_only_deleted_with_joins
+    c1 = ActiveColumnModelWithHasManyRelationship.create(name: 'Jacky')
+    c2 = ActiveColumnModelWithHasManyRelationship.create(name: 'Thomas')
+    p1 = ParanoidModelWithBelongsToActiveColumnModelWithHasManyRelationship.create(name: 'Hello', active_column_model_with_has_many_relationship: c1)
+    
+    c1.destroy
+    assert_equal 1, ActiveColumnModelWithHasManyRelationship.count
+    assert_equal 1, ActiveColumnModelWithHasManyRelationship.only_deleted.count
+    assert_equal 1, ActiveColumnModelWithHasManyRelationship.only_deleted.joins(:paranoid_model_with_belongs_to_active_column_model_with_has_many_relationships).count
+  end
+
   def test_destroy_behavior_for_custom_column_models
     model = CustomColumnModel.new
     assert_equal 0, model.class.count
@@ -181,7 +231,47 @@ class ParanoiaTest < test_framework
   end
 
   def test_default_sentinel_value
-    assert_equal nil, ParanoidModel.paranoia_sentinel_value
+    assert_nil ParanoidModel.paranoia_sentinel_value
+  end
+
+  def test_without_default_scope_option
+    model = WithoutDefaultScopeModel.create
+    model.destroy
+    assert_equal 1, model.class.count
+    assert_equal 1, model.class.only_deleted.count
+    assert_equal 0, model.class.where(deleted_at: nil).count
+  end
+
+  def test_active_column_model
+    model = ActiveColumnModel.new
+    assert_equal 0, model.class.count
+    model.save!
+    assert_nil model.deleted_at
+    assert_equal true, model.active
+    assert_equal 1, model.class.count
+    model.destroy
+
+    assert_equal false, model.deleted_at.nil?
+    assert_nil model.active
+    assert model.paranoia_destroyed?
+
+    assert_equal 0, model.class.count
+    assert_equal 1, model.class.unscoped.count
+    assert_equal 1, model.class.only_deleted.count
+    assert_equal 1, model.class.deleted.count
+  end
+
+  def test_active_column_model_with_uniqueness_validation_only_checks_non_deleted_records
+    a = ActiveColumnModelWithUniquenessValidation.create!(name: "A")
+    a.destroy
+    b = ActiveColumnModelWithUniquenessValidation.new(name: "A")
+    assert b.valid?
+  end
+
+  def test_active_column_model_with_uniqueness_validation_still_works_on_non_deleted_records
+    a = ActiveColumnModelWithUniquenessValidation.create!(name: "A")
+    b = ActiveColumnModelWithUniquenessValidation.new(name: "A")
+    refute b.valid?
   end
 
   def test_sentinel_value_for_custom_sentinel_models
@@ -231,8 +321,7 @@ class ParanoiaTest < test_framework
   # Regression test for #24
   def test_chaining_for_paranoid_models
     scope = FeaturefulModel.where(:name => "foo").only_deleted
-    assert_equal "foo", scope.where_values_hash['name']
-    assert_equal 2, scope.where_values.count
+    assert_equal({'name' => "foo"}, scope.where_values_hash)
   end
 
   def test_only_destroyed_scope_for_paranoid_models
@@ -291,7 +380,7 @@ class ParanoiaTest < test_framework
     model = CallbackModel.new
     model.save
     model.delete
-    assert_equal nil, model.instance_variable_get(:@destroy_callback_called)
+    assert_nil model.instance_variable_get(:@destroy_callback_called)
   end
 
   def test_destroy_behavior_for_callbacks
@@ -331,9 +420,9 @@ class ParanoiaTest < test_framework
     # Just to demonstrate the AR behaviour
     model = NonParanoidModel.new
     model.destroy!
-    assert model.really_destroyed?
+    assert model.destroyed?
     model.destroy!
-    assert model.really_destroyed?
+    assert model.destroyed?
 
     # Mirrors behaviour above
     model = ParanoidModel.new
@@ -453,6 +542,22 @@ class ParanoiaTest < test_framework
     assert RelatedModel.unscoped.exists?(child_2.id)
   end
 
+  def test_really_destroy_behavior_for_callbacks
+    model = CallbackModel.new
+    model.save
+    model.really_destroy!
+
+    assert model.instance_variable_get(:@real_destroy_callback_called)
+  end
+
+  def test_really_destroy_behavior_for_active_column_model
+    model = ActiveColumnModel.new
+    model.save
+    model.really_destroy!
+
+    refute ParanoidModel.unscoped.exists?(model.id)
+  end
+
   def test_really_delete
     model = ParanoidModel.new
     model.save
@@ -488,6 +593,38 @@ class ParanoiaTest < test_framework
     refute c.paranoia_destroyed?
   end
 
+  def test_restore_with_associations_using_recovery_window
+    parent = ParentModel.create
+    first_child = parent.very_related_models.create
+    second_child = parent.very_related_models.create
+
+    parent.destroy
+    second_child.update(deleted_at: parent.deleted_at + 11.minutes)
+
+    parent.restore!(:recursive => true)
+    assert_equal true, parent.deleted_at.nil?
+    assert_equal true, first_child.reload.deleted_at.nil?
+    assert_equal true, second_child.reload.deleted_at.nil?
+
+    parent.destroy
+    second_child.update(deleted_at: parent.deleted_at + 11.minutes)
+
+    parent.restore(:recursive => true, :recovery_window => 10.minutes)
+    assert_equal true, parent.deleted_at.nil?
+    assert_equal true, first_child.reload.deleted_at.nil?
+    assert_equal false, second_child.reload.deleted_at.nil?
+
+    second_child.restore
+    parent.destroy
+    first_child.update(deleted_at: parent.deleted_at - 11.minutes)
+    second_child.update(deleted_at: parent.deleted_at - 9.minutes)
+
+    ParentModel.restore(parent.id, :recursive => true, :recovery_window => 10.minutes)
+    assert_equal true, parent.reload.deleted_at.nil?
+    assert_equal false, first_child.reload.deleted_at.nil?
+    assert_equal true, second_child.reload.deleted_at.nil?
+  end
+
   def test_restore_with_associations
     parent = ParentModel.create
     first_child = parent.very_related_models.create
@@ -685,6 +822,39 @@ class ParanoiaTest < test_framework
     # essentially, we're just ensuring that this doesn't crash
   end
 
+  def test_validates_uniqueness_only_checks_non_deleted_records
+    a = Employer.create!(name: "A")
+    a.destroy
+    b = Employer.new(name: "A")
+    assert b.valid?
+  end
+
+  def test_validates_uniqueness_still_works_on_non_deleted_records
+    a = Employer.create!(name: "A")
+    b = Employer.new(name: "A")
+    refute b.valid?
+  end
+
+  def test_updated_at_modification_on_destroy
+    paranoid_model = ParanoidModelWithTimestamp.create(:parent_model => ParentModel.create, :updated_at => 1.day.ago)
+    assert paranoid_model.updated_at < 10.minutes.ago
+    paranoid_model.destroy
+    assert paranoid_model.updated_at > 10.minutes.ago
+  end
+
+  def test_updated_at_modification_on_restore
+    parent1 = ParentModel.create
+    pt1 = ParanoidModelWithTimestamp.create(:parent_model => parent1)
+    ParanoidModelWithTimestamp.record_timestamps = false
+    pt1.update_columns(created_at: 20.years.ago, updated_at: 10.years.ago, deleted_at: 10.years.ago)
+    ParanoidModelWithTimestamp.record_timestamps = true
+    assert pt1.updated_at < 10.minutes.ago
+    refute pt1.deleted_at.nil?
+    pt1.restore!
+    assert pt1.deleted_at.nil?
+    assert pt1.updated_at > 10.minutes.ago
+  end
+
   def test_i_am_the_destroyer
     expected = %Q{
       Sharon: "There should be a method called I_AM_THE_DESTROYER!"
@@ -791,8 +961,8 @@ class ParanoiaTest < test_framework
     parent_model_with_counter_cache_column = ParentModelWithCounterCacheColumn.create
     related_model = parent_model_with_counter_cache_column.related_models.create
 
-    assert_equal nil, related_model.instance_variable_get(:@after_destroy_callback_called)
-    assert_equal nil, related_model.instance_variable_get(:@after_commit_on_destroy_callback_called)
+    assert_nil related_model.instance_variable_get(:@after_destroy_callback_called)
+    assert_nil related_model.instance_variable_get(:@after_commit_on_destroy_callback_called)
 
     related_model.destroy
 
@@ -800,6 +970,25 @@ class ParanoiaTest < test_framework
     # assert related_model.instance_variable_get(:@after_commit_on_destroy_callback_called)
   end
 
+  def test_uniqueness_for_unparanoid_associated
+    parent_model = ParanoidWithUnparanoids.create
+    related = parent_model.unparanoid_unique_models.create
+    # will raise exception if model is not checked for paranoia
+    related.valid?
+  end
+
+  def test_assocation_not_soft_destroyed_validator
+    notParanoidModel = NotParanoidModelWithBelongsAndAssocationNotSoftDestroyedValidator.create
+    parentModel = ParentModel.create
+    assert notParanoidModel.valid?
+
+    notParanoidModel.parent_model = parentModel
+    assert notParanoidModel.valid?
+    parentModel.destroy
+    assert !notParanoidModel.valid?
+    assert notParanoidModel.errors.full_messages.include? "Parent model has been soft-deleted"
+  end
+
   # TODO: find a fix for Rails 4.1
   if ActiveRecord::VERSION::STRING !~ /\A4\.1/
     def test_counter_cache_column_update_on_really_destroy
@@ -818,14 +1007,52 @@ class ParanoiaTest < test_framework
       parent_model_with_counter_cache_column = ParentModelWithCounterCacheColumn.create
       related_model = parent_model_with_counter_cache_column.related_models.create
 
-      assert_equal nil, related_model.instance_variable_get(:@after_destroy_callback_called)
-      assert_equal nil, related_model.instance_variable_get(:@after_commit_on_destroy_callback_called)
+      assert_nil related_model.instance_variable_get(:@after_destroy_callback_called)
+      assert_nil related_model.instance_variable_get(:@after_commit_on_destroy_callback_called)
 
       related_model.really_destroy!
 
       assert related_model.instance_variable_get(:@after_destroy_callback_called)
       assert related_model.instance_variable_get(:@after_commit_on_destroy_callback_called)
     end
+
+    def test_counter_cache_column_on_double_destroy
+      parent_model_with_counter_cache_column = ParentModelWithCounterCacheColumn.create
+      related_model = parent_model_with_counter_cache_column.related_models.create
+
+      related_model.destroy
+      related_model.destroy
+      assert_equal 0, parent_model_with_counter_cache_column.reload.related_models_count
+    end
+
+    def test_counter_cache_column_on_double_restore
+      parent_model_with_counter_cache_column = ParentModelWithCounterCacheColumn.create
+      related_model = parent_model_with_counter_cache_column.related_models.create
+
+      related_model.destroy
+      related_model.restore
+      related_model.restore
+      assert_equal 1, parent_model_with_counter_cache_column.reload.related_models_count
+    end
+
+    def test_counter_cache_column_on_destroy_and_really_destroy
+      parent_model_with_counter_cache_column = ParentModelWithCounterCacheColumn.create
+      related_model = parent_model_with_counter_cache_column.related_models.create
+
+      related_model.destroy
+      related_model.really_destroy!
+      assert_equal 0, parent_model_with_counter_cache_column.reload.related_models_count
+    end
+
+    def test_counter_cache_column_on_restore
+      parent_model_with_counter_cache_column = ParentModelWithCounterCacheColumn.create
+      related_model = parent_model_with_counter_cache_column.related_models.create
+
+      related_model.destroy
+      assert_equal 0, parent_model_with_counter_cache_column.reload.related_models_count
+      related_model.restore
+      assert_equal 1, parent_model_with_counter_cache_column.reload.related_models_count
+    end
   end
 
   private
@@ -841,11 +1068,27 @@ class ParanoidModel < ActiveRecord::Base
   acts_as_paranoid
 end
 
+class ParanoidWithUnparanoids < ActiveRecord::Base
+  self.table_name = 'plain_models'
+  has_many :unparanoid_unique_models
+end
+
+class UnparanoidUniqueModel < ActiveRecord::Base
+  belongs_to :paranoid_with_unparanoids
+  validates :name, :uniqueness => true
+end
+
 class FailCallbackModel < ActiveRecord::Base
   belongs_to :parent_model
   acts_as_paranoid
 
-  before_destroy { |_| false }
+  before_destroy { |_|
+    if ActiveRecord::VERSION::MAJOR < 5
+      false
+    else
+      throw :abort
+    end
+  }
 end
 
 class FeaturefulModel < ActiveRecord::Base
@@ -853,20 +1096,25 @@ class FeaturefulModel < ActiveRecord::Base
   validates :name, :presence => true, :uniqueness => true
 end
 
+class NonParanoidChildModel < ActiveRecord::Base
+  validates :name, :presence => true, :uniqueness => true
+end
+
 class PlainModel < ActiveRecord::Base
 end
 
 class CallbackModel < ActiveRecord::Base
   acts_as_paranoid
-  before_destroy {|model| model.instance_variable_set :@destroy_callback_called, true }
-  before_restore {|model| model.instance_variable_set :@restore_callback_called, true }
-  before_update  {|model| model.instance_variable_set :@update_callback_called, true }
-  before_save    {|model| model.instance_variable_set :@save_callback_called, true}
+  before_destroy      { |model| model.instance_variable_set :@destroy_callback_called, true }
+  before_restore      { |model| model.instance_variable_set :@restore_callback_called, true }
+  before_update       { |model| model.instance_variable_set :@update_callback_called, true }
+  before_save         { |model| model.instance_variable_set :@save_callback_called, true}
+  before_real_destroy { |model| model.instance_variable_set :@real_destroy_callback_called, true }
 
-  after_destroy  {|model| model.instance_variable_set :@after_destroy_callback_called, true }
-  after_commit   {|model| model.instance_variable_set :@after_commit_callback_called, true }
+  after_destroy       { |model| model.instance_variable_set :@after_destroy_callback_called, true }
+  after_commit        { |model| model.instance_variable_set :@after_commit_callback_called, true }
 
-  validate       {|model| model.instance_variable_set :@validate_called, true }
+  validate            { |model| model.instance_variable_set :@validate_called, true }
 
   def remove_called_variables
     instance_variables.each {|name| (name.to_s.end_with?('_called')) ? remove_instance_variable(name) : nil}
@@ -909,6 +1157,7 @@ end
 
 class Employer < ActiveRecord::Base
   acts_as_paranoid
+  validates_uniqueness_of :name
   has_many :jobs
   has_many :employees, :through => :jobs
 end
@@ -933,6 +1182,86 @@ class CustomSentinelModel < ActiveRecord::Base
   acts_as_paranoid sentinel_value: DateTime.new(0)
 end
 
+class WithoutDefaultScopeModel < ActiveRecord::Base
+  acts_as_paranoid without_default_scope: true
+end
+
+class ActiveColumnModel < ActiveRecord::Base
+  acts_as_paranoid column: :active, sentinel_value: true
+
+  def paranoia_restore_attributes
+    {
+      deleted_at: nil,
+      active: true
+    }
+  end
+
+  def paranoia_destroy_attributes
+    {
+      deleted_at: current_time_from_proper_timezone,
+      active: nil
+    }
+  end
+end
+
+class ActiveColumnModelWithUniquenessValidation < ActiveRecord::Base
+  validates :name, :uniqueness => true
+  acts_as_paranoid column: :active, sentinel_value: true
+
+  def paranoia_restore_attributes
+    {
+      deleted_at: nil,
+      active: true
+    }
+  end
+
+  def paranoia_destroy_attributes
+    {
+      deleted_at: current_time_from_proper_timezone,
+      active: nil
+    }
+  end
+end
+
+class ActiveColumnModelWithHasManyRelationship < ActiveRecord::Base
+  has_many :paranoid_model_with_belongs_to_active_column_model_with_has_many_relationships
+  acts_as_paranoid column: :active, sentinel_value: true
+
+  def paranoia_restore_attributes
+    {
+      deleted_at: nil,
+      active: true
+    }
+  end
+
+  def paranoia_destroy_attributes
+    {
+      deleted_at: current_time_from_proper_timezone,
+      active: nil
+    }
+  end
+end
+
+class ParanoidModelWithBelongsToActiveColumnModelWithHasManyRelationship < ActiveRecord::Base
+  belongs_to :active_column_model_with_has_many_relationship
+
+  acts_as_paranoid column: :active, sentinel_value: true
+
+  def paranoia_restore_attributes
+    {
+      deleted_at: nil,
+      active: true
+    }
+  end
+
+  def paranoia_destroy_attributes
+    {
+      deleted_at: current_time_from_proper_timezone,
+      active: nil
+    }
+  end
+end
+
 class NonParanoidModel < ActiveRecord::Base
 end
 
@@ -990,10 +1319,20 @@ class ParanoidModelWithForeignKeyBelong < ActiveRecord::Base
   belongs_to :paranoid_model_with_has_one
 end
 
+class ParanoidModelWithTimestamp < ActiveRecord::Base
+  belongs_to :parent_model
+  acts_as_paranoid
+end
+
 class NotParanoidModelWithBelong < ActiveRecord::Base
   belongs_to :paranoid_model_with_has_one
 end
 
+class NotParanoidModelWithBelongsAndAssocationNotSoftDestroyedValidator < NotParanoidModelWithBelong
+    belongs_to :parent_model
+    validates :parent_model, association_not_soft_destroyed: true
+end
+
 class FlaggedModel < PlainModel
   acts_as_paranoid :flag_column => :is_deleted
 end
